﻿/*---------------------------------------------------------------------------*
  Project:  Nintendo 3DS CPU Profiler Demo
  File:     runtime_control.cpp

  Copyright (C)2014 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#include <stdlib.h>

#include <nn/os.h>
#include <nn/fnd.h>
#include <nn/prof.h>

#define PROFILE_LOOP_COUNT      100

static int sProfileLoopCount = 0;

static void __declspec(noinline) InterestingFunction()
{
    for (int i = 0; i < 100000; ++i)
    {
        rand();
    }
}

static void __declspec(noinline) UninterestingFunction()
{
    for (int i = 0; i < 100000; ++i)
    {
        rand();
    }
}

extern "C" void nnMain()
{
    srand(nn::fnd::DateTime::GetNow().GetMilliSecond());
    
    nnprofRuntimeControlInitialize();
    nnprofRuntimeControlSetFlags(NN_PROF_FLAGS_CALLSTACKS);
    nnprofRuntimeControlSetSampleMethod(NN_PROF_SAMPLEMETHOD_TIME);
    nnprofRuntimeControlSetSampleTime(NN_PROF_SETTINGS_100X);
    nnprofRuntimeControlSetThreadIDToProfile(nn::os::Thread::GetCurrentId());
    nnprofRuntimeControlSetCoreToProfile(NN_PROF_CORE_APPLICATION);
    
    while(1)
    {
        unsigned int status = nnprofGetProfilerStatus();
        switch (status)
        {                
            case NN_PROFILER_STATUS_NOT_PROFILING:
                sProfileLoopCount = 0;
                NN_LOG("Status == %d\n", status);
                NN_LOG("Status (again) == %d\n", nnprofGetProfilerStatus());
                nnprofRuntimeControlStart();
                break;
                
            case NN_PROFILER_STATUS_IS_PROFILING:
                ++sProfileLoopCount;
                if (sProfileLoopCount > PROFILE_LOOP_COUNT)
                {
                    nnprofRuntimeControlStop();
                }
                break;
        }
        
        nnprofRecordTopMainLoop();
        
        InterestingFunction();
        
        nnprofRuntimeControlDisableProfiling();
        UninterestingFunction();
        nnprofRuntimeControlEnableProfiling();
        
        nn::os::Thread::Sleep(nn::fnd::TimeSpan::FromMicroSeconds(16666));
    }
}
